@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2006 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -45,14 +45,12 @@
 extern "C" {
 #endif
 
-#include <sys/types.h>
-#include <sys/time.h>
-#include <time.h>
 #include <stdarg.h>
+#include <sys/time.h>
+#include <sys/types.h>
 
-#ifdef WIN32
+#if defined(WIN32) && !defined(XINE_COMPILE)
 #include <windows.h>
-#include <windowsx.h>
 #endif
 
 #include <xine/os_types.h>
@@ -144,9 +142,9 @@ void xine_init (xine_t *self) XINE_PROTECTED;
  * and free resources allocated by them
  */
 xine_audio_port_t *xine_open_audio_driver (xine_t *self, const char *id,
-					   void *data) XINE_PROTECTED;
+                                           const void *data) XINE_PROTECTED;
 xine_video_port_t *xine_open_video_driver (xine_t *self, const char *id,
-					   int visual, void *data) XINE_PROTECTED;
+                                           int visual, const void *data) XINE_PROTECTED;
 
 void xine_close_audio_driver (xine_t *self, xine_audio_port_t  *driver) XINE_PROTECTED;
 void xine_close_video_driver (xine_t *self, xine_video_port_t  *driver) XINE_PROTECTED;
@@ -165,6 +163,7 @@ void xine_close_video_driver (xine_t *self, xine_video_port_t  *driver) XINE_PRO
 #define XINE_VISUAL_TYPE_MACOSX            9
 #define XINE_VISUAL_TYPE_XCB              11
 #define XINE_VISUAL_TYPE_RAW              12
+#define XINE_VISUAL_TYPE_WAYLAND          13
 
 /*
  * free all resources, close all plugins, close engine.
@@ -191,6 +190,14 @@ void xine_exit (xine_t *self) XINE_PROTECTED;
 xine_stream_t *xine_stream_new (xine_t *self,
 				xine_audio_port_t *ao, xine_video_port_t *vo) XINE_PROTECTED;
 
+/*
+ * A side stream is like a side car on a bike. It has its own mrl, input, and demux.
+ * Everything else is handled by the master (play, seek, pause, close, dispose).
+ * This is intended for streams who have separate mrls for audio, video, and subtitle.
+ * Index 0 just returns the master itself, 1 ... 3 are free for side mrls. */
+#define XINE_SIDE_STREAMS 1
+xine_stream_t *xine_get_side_stream (xine_stream_t *master, int index) XINE_PROTECTED;
+
 /*
  * Make one stream the slave of another.
  * This establishes a binary master slave relation on streams, where
@@ -230,6 +237,35 @@ int xine_stream_master_slave(xine_stream_t *master, xine_stream_t *slave,
  */
 int xine_open (xine_stream_t *stream, const char *mrl) XINE_PROTECTED;
 
+/** The keyframe seek index feature. */
+
+#define XINE_KEYFRAMES 1 /**<< Check this for feature available. */
+
+typedef struct {
+  int msecs;   /**<< Milliseconds from beginning of stream. */
+  int normpos; /**<< Size based stream position (beginning == 0, end == 64k - 1). */
+} xine_keyframes_entry_t;
+
+/** @brief Query stream keyframe seek index.
+    @note  We dont do an expensive file scan. We will only find keyframes listed in
+           container or already seen while playing.
+    @param stream The stream that index is for.
+    @param pos    On call, the start time or normpos.
+                  On return, the found time and normpos.
+    @param offs   0 gets nearest keyframe, other values step from exact given pos.
+    @return  0: Found.
+             1: Search truncated to first or last known keyframe.
+             2: Failure.
+*/
+int xine_keyframes_find (xine_stream_t *stream, xine_keyframes_entry_t *pos, int offs) XINE_PROTECTED;
+
+/** @brief Get a private stream keyframe seek index copy, free () it when done.
+    @param stream The stream that index is for.
+    @param size   On return, the count of entries there are.
+    @return The entry array or NULL.
+*/
+xine_keyframes_entry_t *xine_keyframes_get (xine_stream_t *stream, int *size) XINE_PROTECTED;
+
 /*
  * play a stream from a given position
  *
@@ -355,6 +391,7 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
 #define XINE_PARAM_VO_CROP_RIGHT           0x01000021 /* crop frame pixels  */
 #define XINE_PARAM_VO_CROP_TOP             0x01000022 /* crop frame pixels  */
 #define XINE_PARAM_VO_CROP_BOTTOM          0x01000023 /* crop frame pixels  */
+#define XINE_PARAM_VO_SINGLE_STEP          0x01000024 /* 1 = "advance to next frame, then pause" */
 
 #define XINE_VO_ZOOM_STEP                  100
 #define XINE_VO_ZOOM_MAX                   400
@@ -419,7 +456,7 @@ int  xine_get_param (xine_stream_t *stream, int param) XINE_PROTECTED;
 int  xine_get_current_frame (xine_stream_t *stream,
 			     int *width, int *height,
 			     int *ratio_code, int *format,
-			     uint8_t *img) XINE_PROTECTED;
+                             uint8_t *img) XINE_PROTECTED XINE_DEPRECATED;
 
 int  xine_get_current_frame_s (xine_stream_t *stream,
 			     int *width, int *height,
@@ -431,8 +468,9 @@ int  xine_get_current_frame_alloc (xine_stream_t *stream,
 			     int *ratio_code, int *format,
 			     uint8_t **img, int *img_size) XINE_PROTECTED;
 
-typedef struct {
+typedef struct xine_current_frame_data_s xine_current_frame_data_t;
 
+struct xine_current_frame_data_s {
   int      width;
   int      height;
   int      crop_left;
@@ -444,7 +482,7 @@ typedef struct {
   int      format;
   int      img_size;
   uint8_t *img;
-} xine_current_frame_data_t;
+};
 
 #define XINE_FRAME_DATA_ALLOCATE_IMG (1<<0)
 
@@ -458,6 +496,7 @@ int  xine_get_current_frame_data (xine_stream_t *stream,
 #define XINE_IMGFMT_XVMC (('C'<<24)|('M'<<16)|('v'<<8)|'X')
 #define XINE_IMGFMT_XXMC (('C'<<24)|('M'<<16)|('x'<<8)|'X')
 #define XINE_IMGFMT_VDPAU (('A'<<24)|('P'<<16)|('D'<<8)|'V')
+#define XINE_IMGFMT_VAAPI (('P'<<24)|('A'<<16)|('A'<<8)|'V')
 
 /* get current xine's virtual presentation timestamp (1/90000 sec)
  * note: this is mostly internal data.
@@ -835,7 +874,7 @@ typedef struct {
    * method to set all the read/write parameters.
    * params is a struct * defined by xine_post_api_descr_t
    */
-  int (*set_parameters) (xine_post_t *self, void *params);
+  int (*set_parameters) (xine_post_t *self, const void *params);
 
   /*
    * method to get all parameters.
@@ -1063,13 +1102,15 @@ const char *xine_get_meta_info   (xine_stream_t *stream, int info) XINE_PROTECTE
  *       and should never be free()d.
  */
 
-typedef struct {
+typedef struct xine_mrl_s xine_mrl_t;
+
+struct xine_mrl_s {
   char      *origin; /* file plugin: path */
   char      *mrl;    /* <type>://<location> */
   char      *link;
   off_t      size;   /* size of this source, may be 0 */
   uint32_t   type;   /* see below */
-} xine_mrl_t;
+};
 
 /* mrl types */
 #define XINE_MRL_TYPE_unknown        (0 << 0)
@@ -1364,6 +1405,31 @@ typedef struct {
 
 } xcb_visual_t;
 
+/*
+ * this is the visual data struct any Wayland GUI
+ * must supply to the xine_open_video_driver call
+ * ("data" parameter)
+ */
+
+struct wl_display;
+struct wl_surface;
+
+typedef struct {
+
+  struct wl_display *display;
+  struct wl_surface *surface;
+
+  void *user_data;
+  void (*frame_output_cb) (void *user_data,
+                           int video_width, int video_height,
+                           double video_pixel_aspect,
+                           int *dest_x, int *dest_y,
+                           int *dest_width, int *dest_height,
+                           double *dest_pixel_aspect,
+                           int *win_x, int *win_y);
+
+} xine_wayland_visual_t;
+
 /**************************************************
  * XINE_VO_RAW struct definitions
  *************************************************/
@@ -1451,7 +1517,7 @@ typedef struct {
 
 } fb_visual_t;
 
-#ifdef WIN32
+#if defined(WIN32) && (!defined(XINE_COMPILE) || defined(XINE_NEED_WIN32_VISUAL))
 /*
  * this is the visual data struct any win32 gui should supply
  * (pass this to init_video_out_plugin or the xine_load_video_output_plugin
@@ -1669,6 +1735,17 @@ int   xine_config_register_bool   (xine_t *self,
 				   xine_config_cb_t changed_cb,
 				   void *cb_data) XINE_PROTECTED;
 
+/**
+ * unregister multiple entry callback functions.
+ * all 3 values need to match unless they are NULL.
+ * if cb_data_size is not zero, data pointers within the range
+ * (cb_data <= ptr < cb_data + cb_data_size) will match.
+ * returns the count of unregistered functions.
+ */
+#define HAVE_XINE_CONFIG_UNREGISTER_CALLBACKS 1
+int xine_config_unregister_callbacks (xine_t *self,
+  const char *key, xine_config_cb_t changed_cb, void *cb_data, size_t cb_data_size) XINE_PROTECTED;
+
 /*
  * the following functions will copy data from the internal xine_config
  * data database to the xine_cfg_entry_t *entry you provide
@@ -1686,6 +1763,20 @@ int  xine_config_get_next_entry (xine_t *self, xine_cfg_entry_t *entry) XINE_PRO
 int  xine_config_lookup_entry (xine_t *self, const char *key,
 			       xine_cfg_entry_t *entry) XINE_PROTECTED;
 
+/*
+ * Helper function to get numeric value of config entry.
+ * Return def_value if entry was not found (or is not correct type).
+ */
+int xine_config_lookup_num(xine_t *self, const char *key, int def_value) XINE_PROTECTED;
+
+/*
+ * Thread-safe helper function to get string value of config entry.
+ * Return copy of current value or NULL.
+ * Returned string must be freed with xine_config_free_str().
+ */
+char *xine_config_lookup_string(xine_t *self, const char *key) XINE_PROTECTED;
+void xine_config_free_string(xine_t *self, char **value) XINE_PROTECTED;
+
 /*
  * update a config entry (which was returned from lookup_entry() )
  *
@@ -2140,14 +2231,24 @@ typedef struct xine_event_queue_s xine_event_queue_t;
 xine_event_queue_t *xine_event_new_queue (xine_stream_t *stream) XINE_PROTECTED;
 void xine_event_dispose_queue (xine_event_queue_t *queue) XINE_PROTECTED;
 
+/** @brief Filter events by type.
+ *  @param queue The queue that shall receive a few types only.
+ *  @param types List of event types, termiated by XINE_EVENT_QUIT. NULL reverts to the "all types" default. */
+void xine_event_select (xine_event_queue_t *queue, const int *types) XINE_PROTECTED;
+
 /*
  * receive events (poll)
  *
  * use xine_event_free on the events received from these calls
  * when they're no longer needed
  */
+/* get first event or NULL when queue is empty. */
 xine_event_t *xine_event_get  (xine_event_queue_t *queue) XINE_PROTECTED;
+/* free prev_event if not NULL, then same as xine_event_get (). */
+xine_event_t *xine_event_next (xine_event_queue_t *queue, xine_event_t *prev_event) XINE_PROTECTED;
+/* get first event, wait for it if needed. */
 xine_event_t *xine_event_wait (xine_event_queue_t *queue) XINE_PROTECTED;
+/* free or reuse event. may be NULL. */
 void          xine_event_free (xine_event_t *event) XINE_PROTECTED;
 
 /*
@@ -2157,12 +2258,15 @@ void          xine_event_free (xine_event_t *event) XINE_PROTECTED;
  * the specified queue, call your callback on each of them
  * and will then free the event when your callback returns
  *
+ * Note: only one listener thread / event queue !
+ *
+ * @return 0 on error
  */
 typedef void (*xine_event_listener_cb_t) (void *user_data,
 					  const xine_event_t *event);
-void xine_event_create_listener_thread (xine_event_queue_t *queue,
-					xine_event_listener_cb_t callback,
-					void *user_data) XINE_PROTECTED;
+int xine_event_create_listener_thread (xine_event_queue_t *queue,
+                                       xine_event_listener_cb_t callback,
+                                       void *user_data) XINE_PROTECTED;
 
 /*
  * send an event to all queues
@@ -2173,6 +2277,7 @@ void xine_event_create_listener_thread (xine_event_queue_t *queue,
 void xine_event_send (xine_stream_t *stream, const xine_event_t *event) XINE_PROTECTED;
 
 
+
 /*********************************************************************
  * OSD (on screen display)                                           *
  *********************************************************************/
