@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -21,16 +21,14 @@
 #ifndef HAVE_INPUT_PLUGIN_H
 #define HAVE_INPUT_PLUGIN_H
 
-#include <sys/types.h>
+#include <sys/types.h>  /* off_t */
 
+#include <xine/attributes.h>
 #include <xine/os_types.h>
 #include <xine/xineutils.h>
 #include <xine/buffer.h>
-#include <xine/configfile.h>
 
-#ifdef XINE_COMPILE
-#  include <xine/plugin_catalog.h>
-#endif
+struct plugin_node_s;
 
 #define INPUT_PLUGIN_IFACE_VERSION   18
 
@@ -120,7 +118,7 @@ struct input_plugin_s {
    * Should block until some bytes available for read;
    * a return value of 0 indicates no data available
    */
-  off_t (*read) (input_plugin_t *this_gen, void *buf, off_t nlen);
+  off_t (*read) (input_plugin_t *this_gen, void *buf, off_t nlen) XINE_USED;
 
 
   /*
@@ -145,7 +143,8 @@ struct input_plugin_s {
    *
    * if seeking failed, -1 is returned
    *
-   * note: only SEEK_SET (0) is currently supported as origin
+   * note: only SEEK_SET (0) is currently supported as origin,
+   *       unless INPUT_CAP_TIME_SEEKABLE is set.
    * note: may be NULL is not supported
    */
   off_t (*seek_time) (input_plugin_t *this_gen, int time_offset, int origin);
@@ -231,12 +230,7 @@ struct input_plugin_s {
    * Used by the plugins loader. It's an opaque type when using the
    * structure outside of xine's build.
    */
-#ifdef XINE_COMPILE
-  plugin_node_t *node;
-#else
-  void *node;
-#endif
-
+  struct plugin_node_s *node XINE_PRIVATE_FIELD;
 };
 
 /*
@@ -317,8 +311,54 @@ struct input_plugin_s {
  */
 #define INPUT_CAP_NO_CACHE             0x00000200
 
+/*
+ * INPUT_CAP_CLONE:
+ *   open a second input plugin instance on the same source.
+ */
+#define INPUT_CAP_CLONE                0x00000400
+
+/*
+ * INPUT_CAP_SLOW_SEEKABLE:
+ *   Like INPUT_CAP_SEEKABLE, but seeking may be expensive.
+ *   Short forward seeks are fast (ex. implemented by skipping data).
+ *   Seeking back or far forward is expensive.
+ *
+ *   This type of seeking should be used only when user issues seek request
+ *   and the destination offset is known (no binary search etc. required).
+ *   Another use case could be when header is located at the end of file.
+ *
+ */
+#define INPUT_CAP_SLOW_SEEKABLE        0x00000800
 
 #define INPUT_IS_SEEKABLE(input) (((input)->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0)
+#define INPUT_IS_SLOW_SEEKABLE(input) (((input)->get_capabilities(input) & (INPUT_CAP_SEEKABLE | INPUT_CAP_SLOW_SEEKABLE)) != 0)
+
+/*
+ * INPUT_CAP_SIZED_PREVIEW:
+ *   Like INPUT_CAP_PREVIEW, but the buffer shall contain an int telling
+ *   the count of bytes to read before the call.
+ */
+#define INPUT_CAP_SIZED_PREVIEW        0x00001000
+
+/*
+ * INPUT_CAP_TIME_SEEKABLE:
+ *   Time based seek works reliably and should be preferred over size based seek.
+ */
+#define INPUT_CAP_TIME_SEEKABLE        0x00002000
+
+/*
+ * INPUT_CAP_NEW_MRL:
+ *   Plugin can try to switch to a new mrl on the fly.
+ *   See INPUT_OPTIONAL_DATA_NEW_MRL below.
+ */
+#define INPUT_CAP_NEW_MRL              0x00004000
+
+/*
+ * INPUT_CAP_LIVE:
+ *   Plugin knows it has a real live stream. It runs at a fixed speed that is
+ *   _roughly_ the same as XINE_FINE_SPEED_NORMAL.
+ */
+#define INPUT_CAP_LIVE                  0x00008000
 
 #define INPUT_OPTIONAL_UNSUPPORTED    0
 #define INPUT_OPTIONAL_SUCCESS        1
@@ -333,10 +373,26 @@ struct input_plugin_s {
 #define INPUT_OPTIONAL_DATA_DEMUX_MIME_TYPE 9
 /* buffer is a const char **; the string is static or freed by the input plugin. */
 #define INPUT_OPTIONAL_DATA_DEMUXER   10
+/* buffer is a struct input_plugin_s **; release by calling ptr->dispose (ptr). */
+#define INPUT_OPTIONAL_DATA_CLONE     11
+/* see INPUT_CAP_SIZED_PREVIEW above. */
+#define INPUT_OPTIONAL_DATA_SIZED_PREVIEW 12
+/* buffer is an int32_t * where input plugin will store media duration in milliseconds. */
+#define INPUT_OPTIONAL_DATA_DURATION  13
+/* buffer is a const char * holding the new mrl to try.
+ * fragment streams can avoid input_plugin->dispose () followed by input_class->get_instance (),
+ * or _x_free_input_plugin () followed by _x_find_input_plugin () on a lot of * similar mrls. */
+#define INPUT_OPTIONAL_DATA_NEW_MRL   14
 
 #define MAX_MRL_ENTRIES 255
 #define MAX_PREVIEW_SIZE 4096
 
+/* network buffering control */
+typedef struct xine_nbc_st xine_nbc_t;
+xine_nbc_t *xine_nbc_init (xine_stream_t *stream) XINE_PROTECTED;
+void xine_nbc_close (xine_nbc_t *nbc) XINE_PROTECTED;
+
+
 /* Types of mrls returned by get_dir() */
 #define mrl_unknown        (0 << 0)
 #define mrl_dvd            (1 << 0)
