@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2021 the xine project
+ * Copyright (C) 2004-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -167,7 +167,7 @@ static int read_iff_chunk(demux_iff_t *this) {
   unsigned char signature[IFF_SIGNATURE_SIZE];
   unsigned char buffer[512];
   unsigned int  keep_on_reading = 1;
-  uint32_t      junk_size, junk_type;
+  uint32_t      junk_size;
 
   while ( keep_on_reading == 1 ) {
     if (this->input->read(this->input, signature, IFF_JUNK_SIZE) == IFF_JUNK_SIZE) {
@@ -182,9 +182,8 @@ static int read_iff_chunk(demux_iff_t *this) {
         if (this->input->read(this->input, &signature[7], 1) != 1)
           return 0;
       }
-      junk_type = _X_BE_32(&signature[0]);
       junk_size = _X_BE_32(&signature[4]);
-      switch (junk_type) {
+      switch( _X_BE_32(&signature[0]) ) {
         case IFF_CMAP_CHUNK:
         case IFF_BODY_CHUNK:
         case IFF_DLTA_CHUNK:
@@ -202,7 +201,7 @@ static int read_iff_chunk(demux_iff_t *this) {
           break;
       }
 
-      switch (junk_type) {
+      switch( _X_BE_32(&signature[0]) ) {
         case IFF_FORM_CHUNK:
           if (this->input->read(this->input, buffer, 4) != 4)
             return 0;
