@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -49,8 +49,7 @@
 #include <xine/buffer.h>
 #include "video_out_dxr3.h"
 #include "dxr3.h"
-
-#include "compat.c"
+#include "group_dxr3.h"
 
 /* once activated, we wait for this amount of missing pan&scan info
  * before disabling it again */
@@ -66,26 +65,6 @@
 /* the number of frames to pass before we stop duration correction */
 #define FORCE_DURATION_WINDOW_SIZE 100
 
-
-/* plugin class initialization function */
-static void     *dxr3_init_plugin(xine_t *xine, void *);
-
-
-/* plugin catalog information */
-static const uint32_t supported_types[] = { BUF_VIDEO_MPEG, 0 };
-
-static const decoder_info_t dxr3_video_decoder_info = {
-  supported_types,     /* supported types */
-  10                   /* priority        */
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_VIDEO_DECODER, 19, "dxr3-mpeg2", XINE_VERSION_CODE, &dxr3_video_decoder_info, &dxr3_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
-
 /* plugin class functions */
 static video_decoder_t *dxr3_open_plugin(video_decoder_class_t *class_gen, xine_stream_t *stream);
 
@@ -101,8 +80,6 @@ typedef struct dxr3_decoder_class_s {
   video_decoder_class_t  video_decoder_class;
 
   int                    instance;             /* we allow only one instance of this plugin */
-
-  metronom_clock_t      *clock;                /* used for syncing */
 } dxr3_decoder_class_t;
 
 typedef struct dxr3_decoder_s {
@@ -111,6 +88,8 @@ typedef struct dxr3_decoder_s {
   xine_stream_t         *stream;
   dxr3_scr_t            *scr;                  /* shortcut to the scr plugin in the dxr3 video out */
 
+  metronom_clock_t      *clock;                /* used for syncing */
+
   int                    devnum;
   int                    fd_control;
   int                    fd_video;             /* to access the dxr3 devices */
@@ -174,10 +153,13 @@ static inline int dxr3_mvcommand(int fd_control, int command)
 }
 
 
-static void *dxr3_init_plugin(xine_t *xine, void *data)
+void *dxr3_video_init_plugin(xine_t *xine, const void *data)
 {
   dxr3_decoder_class_t *this;
 
+  (void)xine;
+  (void)data;
+
   this = calloc(1, sizeof (dxr3_decoder_class_t));
   if (!this) return NULL;
 
@@ -188,8 +170,6 @@ static void *dxr3_init_plugin(xine_t *xine, void *data)
 
   this->instance                            = 0;
 
-  this->clock                               = xine->clock;
-
   return &this->video_decoder_class;
 }
 
@@ -219,6 +199,7 @@ static video_decoder_t *dxr3_open_plugin(video_decoder_class_t *class_gen, xine_
   this->class                       = class;
   this->stream                      = stream;
   this->scr                         = NULL;
+  this->clock                       = stream->xine->clock;
 
   this->devnum = cfg->register_num(cfg, CONF_KEY, 0, CONF_NAME, CONF_HELP, 10, NULL, NULL);
 
@@ -253,7 +234,7 @@ static video_decoder_t *dxr3_open_plugin(video_decoder_class_t *class_gen, xine_
   this->dts_offset[2]         = 21600;
 
   this->force_duration_window = -FORCE_DURATION_WINDOW_SIZE;
-  this->last_vpts             = this->class->clock->get_current_time(this->class->clock);
+  this->last_vpts             = this->clock->get_current_time(this->clock);
 
   this->sync_every_frame      = cfg->register_bool(cfg,
     "dxr3.playback.sync_every_frame", 0, _("try to sync video every frame"),
@@ -503,7 +484,7 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
    * also ensure the scr is running
    */
   if (this->fd_video < 0) {
-    metronom_clock_t *clock = this->class->clock;
+    metronom_clock_t *clock = this->clock;
     char tmpstr[128];
     int64_t time;
 
@@ -522,7 +503,7 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
      * driver bug and gets fixed somewhen. FIXME: We might then
      * want to do this entirely in the video out.
      */
-    this->scr = ((dxr3_driver_t *)this->stream->video_driver)->class->scr;
+    this->scr = ((dxr3_driver_t *)this->stream->video_driver)->scr;
     time = clock->get_current_time(clock);
     this->scr->scr_plugin.start(&this->scr->scr_plugin, time);
     clock->register_scr(clock, &this->scr->scr_plugin);
@@ -548,8 +529,7 @@ static void dxr3_decode_data(video_decoder_t *this_gen, buf_element_t *buf)
     }
     vpts -= this->dts_offset[2];
 
-    delay = vpts - this->class->clock->get_current_time(
-      this->class->clock);
+    delay = vpts - this->clock->get_current_time(this->clock);
     llprintf(LOG_PTS, "SETPTS got %" PRId64 "\n", vpts);
     /* SETPTS only if less then one second in the future and
      * either buffer has pts or sync_every_frame is set */
@@ -601,6 +581,7 @@ static void dxr3_reset(video_decoder_t *this_gen)
 
 static void dxr3_discontinuity(video_decoder_t *this_gen)
 {
+  (void)this_gen;
 }
 
 static void dxr3_flush(video_decoder_t *this_gen)
@@ -626,7 +607,7 @@ static void dxr3_flush(video_decoder_t *this_gen)
 static void dxr3_dispose(video_decoder_t *this_gen)
 {
   dxr3_decoder_t *this = (dxr3_decoder_t *)this_gen;
-  metronom_clock_t *clock = this->class->clock;
+  metronom_clock_t *clock = this->clock;
 
   if (this->scr)
     clock->unregister_scr(clock, &this->scr->scr_plugin);
