@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2019 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -90,7 +90,7 @@
 #define	DEFAULT_CDDA_DEVICE	"/dev/cdrom"
 #endif
 
-#define CDDB_SERVER             "freedb.freedb.org"
+#define CDDB_SERVER             "gnudb.gnudb.org"
 #define CDDB_PORT               8880
 #define CDDB_PROTOCOL           6
 #define CDDB_TIMEOUT            5000
@@ -782,7 +782,7 @@ static cdrom_toc_t *read_cdrom_toc (int fd) {
 
   /* allocate space for the toc entries */
   toc = calloc (1, sizeof (cdrom_toc_t) + total_tracks * sizeof (cdrom_toc_entry_t));
-  if (!toc->toc_entries) {
+  if (!toc) {
     perror("calloc");
     return NULL;
   }
@@ -874,7 +874,7 @@ static cdrom_toc_t *read_cdrom_toc (int fd) {
   toc->toc_entries[i].track_mode = (tocentry.data->control & 0x04) ? 1 : 0;
   toc->toc_entries[i].first_frame_minute = tocentry.data->addr.msf.minute;
   toc->toc_entries[i].first_frame_second = tocentry.data->addr.msf.second;
-  toc->toc_entries[i]..first_frame_frame = tocentry.data->addr.msf.frame;
+  toc->toc_entries[i].first_frame_frame = tocentry.data->addr.msf.frame;
   toc->toc_entries[i].first_frame =
     (tocentry.data->addr.msf.minute * CD_SECONDS_PER_MINUTE * CD_FRAMES_PER_SECOND) +
     (tocentry.data->addr.msf.second * CD_FRAMES_PER_SECOND) +
@@ -1177,12 +1177,13 @@ network_command( xine_stream_t *stream, int socket, void *data_buf, const char *
 #ifndef WIN32
 static int network_connect(xine_stream_t *stream, const char *got_url )
 {
-  char *host;
+  char *host = NULL;
   int port;
   int fd;
 
   char *url = strdup(got_url);
-  parse_url(url, &host, &port);
+  if (url)
+    parse_url(url, &host, &port);
 
   if( !host || !strlen(host) || !port )
   {
@@ -1438,7 +1439,9 @@ static void _cdda_mkdir_recursive_safe (xine_t *xine, char *path)
 
   char buf[strlen (path) + 1];
   strcpy (buf, path);
-  char *p = strchr (buf, '/') ? : buf;
+  char *p = strchr (buf, '/');
+  if (!p)
+    p = buf;
 
   do
   {
