@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2017 the xine project
+ * Copyright (C) 2012-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -45,6 +45,8 @@
 #include <xine/input_plugin.h>
 #include <xine/video_out.h>
 
+#include "input_helper.h"
+
 /* describe tests here */
 static const char * const test_names[] = {
   "test://",
@@ -230,9 +232,14 @@ static int test_make (test_input_plugin_t * this) {
   /* dimensions */
   width = 320;
   if (this->stream && this->stream->video_out) {
-    x = this->stream->video_out->get_property (this->stream->video_out,
-      VO_PROP_WINDOW_WIDTH);
-    if (x > width) width = x;
+    if (_x_lock_port_rewiring(this->stream->xine, 0)) {
+      if (this->stream->video_out) {
+        x = this->stream->video_out->get_property (this->stream->video_out,
+                                                   VO_PROP_WINDOW_WIDTH);
+        if (x > width) width = x;
+      }
+      _x_unlock_port_rewiring(this->stream->xine);
+    }
   }
   if (width > 1920) width = 1920;
   width &= ~1;
@@ -553,10 +560,6 @@ static int test_make (test_input_plugin_t * this) {
 
 /* instance functions */
 
-static uint32_t test_plugin_get_capabilities (input_plugin_t *this_gen) {
-  return INPUT_CAP_SEEKABLE;
-}
-
 static off_t test_plugin_read (input_plugin_t *this_gen, void *buf, off_t len) {
   test_input_plugin_t *this = (test_input_plugin_t *) this_gen;
 
@@ -564,7 +567,7 @@ static off_t test_plugin_read (input_plugin_t *this_gen, void *buf, off_t len) {
   if (len > this->filesize - this->filepos) len = this->filesize - this->filepos;
 
   if (test_is_yuv[this->index]) {
-    char *p = this->y4m_frame, *q = buf;
+    unsigned char *p = this->y4m_frame, *q = buf;
     off_t l = len, d;
     d = this->headsize - this->filepos;
     if (d > 0) {
@@ -594,39 +597,16 @@ static off_t test_plugin_read (input_plugin_t *this_gen, void *buf, off_t len) {
   return len;
 }
 
-static buf_element_t *test_plugin_read_block (input_plugin_t *this_gen, fifo_buffer_t *fifo,
-  off_t todo) {
-  test_input_plugin_t *this = (test_input_plugin_t *) this_gen;
-  buf_element_t *buf;
-
-  if (!this->buf || (todo < 0)) return NULL;
-
-  buf = fifo->buffer_pool_alloc (fifo);
-  if (todo > buf->max_size) todo = buf->max_size;
-  buf->type = BUF_DEMUX_BLOCK;
-  test_plugin_read (this_gen, buf->content, todo);
-
-  return buf;
-}
-
 static off_t test_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin) {
   test_input_plugin_t *this = (test_input_plugin_t *) this_gen;
-  off_t newpos = offset;
 
-  switch (origin) {
-    case SEEK_SET: break;
-    case SEEK_CUR: newpos += this->filepos; break;
-    case SEEK_END: newpos += this->filesize; break;
-    default: newpos = -1;
+  offset = _x_input_translate_seek(offset, origin, this->filepos, this->filesize);
+
+  if (offset >= 0) {
+    this->filepos = offset;
   }
 
-  if ((newpos < 0) || (newpos > this->filesize)) {
-    errno = EINVAL;
-    return (off_t)-1;
-  }
-
-  this->filepos = newpos;
-  return newpos;
+  return offset;
 }
 
 static off_t test_plugin_get_current_pos (input_plugin_t *this_gen) {
@@ -641,22 +621,12 @@ static off_t test_plugin_get_length (input_plugin_t *this_gen) {
   return this->filesize;
 }
 
-static uint32_t test_plugin_get_blocksize (input_plugin_t *this_gen) {
-  return 0;
-}
-
 static const char *test_plugin_get_mrl (input_plugin_t *this_gen) {
   test_input_plugin_t *this = (test_input_plugin_t *) this_gen;
 
   return test_names[this->index];
 }
 
-static int test_plugin_get_optional_data (input_plugin_t *this_gen, void *data,
-  int data_type) {
-
-  return INPUT_OPTIONAL_UNSUPPORTED;
-}
-
 static void test_plugin_dispose (input_plugin_t *this_gen ) {
   test_input_plugin_t *this = (test_input_plugin_t *) this_gen;
 
@@ -673,7 +643,7 @@ static int test_plugin_open (input_plugin_t *this_gen ) {
 static input_plugin_t *test_class_get_instance (input_class_t *cls_gen,
   xine_stream_t *stream, const char *data) {
   test_input_plugin_t *this;
-  int i;
+  unsigned int i;
 
   for (i = 0; i < TEST_FILES; i++) {
     if (!strcasecmp (data, test_names[i])) break;
@@ -681,19 +651,22 @@ static input_plugin_t *test_class_get_instance (input_class_t *cls_gen,
   if (i == TEST_FILES) return NULL;
 
   this = (test_input_plugin_t *) calloc(1, sizeof (test_input_plugin_t));
+  if (!this)
+    return NULL;
+
   this->stream = stream;
   this->index = i;
 
   this->input_plugin.open               = test_plugin_open;
-  this->input_plugin.get_capabilities   = test_plugin_get_capabilities;
+  this->input_plugin.get_capabilities   = _x_input_get_capabilities_seekable;
   this->input_plugin.read               = test_plugin_read;
-  this->input_plugin.read_block         = test_plugin_read_block;
+  this->input_plugin.read_block         = _x_input_default_read_block;
   this->input_plugin.seek               = test_plugin_seek;
   this->input_plugin.get_current_pos    = test_plugin_get_current_pos;
   this->input_plugin.get_length         = test_plugin_get_length;
-  this->input_plugin.get_blocksize      = test_plugin_get_blocksize;
+  this->input_plugin.get_blocksize      = _x_input_default_get_blocksize;
   this->input_plugin.get_mrl            = test_plugin_get_mrl;
-  this->input_plugin.get_optional_data  = test_plugin_get_optional_data;
+  this->input_plugin.get_optional_data  = _x_input_default_get_optional_data;
   this->input_plugin.dispose            = test_plugin_dispose;
   this->input_plugin.input_class        = cls_gen;
 
@@ -706,6 +679,7 @@ static input_plugin_t *test_class_get_instance (input_class_t *cls_gen,
  */
 
 static const char * const * test_get_autoplay_list (input_class_t *this_gen, int *num_files) {
+  (void)this_gen;
   if (num_files) *num_files = TEST_FILES - 1;
   return test_names + 1;
 }
@@ -713,9 +687,10 @@ static const char * const * test_get_autoplay_list (input_class_t *this_gen, int
 static xine_mrl_t **test_class_get_dir (input_class_t *this_gen, const char *filename,
   int *nFiles) {
   test_input_class_t *this = (test_input_class_t *) this_gen;
-  int i;
+  unsigned int i;
   xine_mrl_t *m;
 
+  (void)filename;
   if (!this->mrls[0]) {
     for (i = 0; i < TEST_FILES - 1; i++) {
       m = &this->m[i];
@@ -740,10 +715,13 @@ static void test_class_dispose (input_class_t *this_gen) {
   free (this_gen);
 }
 
-static void *test_init_plugin (xine_t *xine, void *data) {
+static void *test_init_plugin (xine_t *xine, const void *data) {
   test_input_class_t *this;
 
+  (void)data;
   this = (test_input_class_t *) calloc(1, sizeof (test_input_class_t));
+  if (!this)
+    return NULL;
 
   this->xine   = xine;
 
@@ -759,7 +737,7 @@ static void *test_init_plugin (xine_t *xine, void *data) {
 }
 
 static const input_info_t input_info_test = {
-  110                       /* priority */
+  .priority = 110
 };
 
 /*
@@ -772,6 +750,7 @@ static const input_info_t input_info_test = {
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   INPUT_TEST_CATALOG,
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 #endif
+
