@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2018 the xine project
+ * Copyright (C) 2000-2020 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -25,8 +25,6 @@
 #include "config.h"
 #endif
 
-#include <config.h>
-
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -435,7 +433,7 @@ static void rtsp_basicauth (const char *user, const char *password, char** dest)
   const size_t enclen = ((totlen + 2) * 4 ) / 3 + 12;
   char         tmp[totlen + 4];
 
-  snprintf(tmp, totlen + 1, "%s:%s", user, password ? : "");
+  snprintf(tmp, totlen + 1, "%s:%s", user, password ? password : "");
 
   *dest = malloc(enclen);
   xine_base64_encode ((unsigned char *)tmp, *dest, totlen);
