@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2009 the xine project
+ * Copyright (C) 2002-2017 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -55,7 +55,7 @@ int rtsp_send_ok(rtsp_t *s);
 
 int rtsp_read_data(rtsp_t *s, void *buffer, unsigned int size);
 
-char* rtsp_search_answers(rtsp_t *s, const char *tag);
+const char* rtsp_search_answers(rtsp_t *s, const char *tag);
 void rtsp_add_to_payload(char **payload, const char *string);
 
 void rtsp_free_answers(rtsp_t *this);
