@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2004 the xine project
+ * Copyright (C) 2000-2019 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -24,6 +24,10 @@
  * real work by goom author, JC Hoelt <jeko@free.fr>.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
@@ -35,15 +39,14 @@
 */
 
 /* TJ. this is what my 32bit Linux running on a AMD Athlon II X2 240e (2 * 2800Mhz)
-  performed @ 1024x576, 19 fps:
-  goom: csc_method 0 min 5355 us avg 5398 us
-  goom: csc_method 1 min 12718 us avg 12855 us
-  goom: csc_method 2 min 4199 us avg 4289 us
+  performed @ 1280x720, 19 fps:
+  goom: csc_method 0 min 8257 us avg 8308 us
+  goom: csc_method 1 min 16312 us avg 16413 us
+  goom: csc_method 2 min 5019 us avg 5134 us
   Seems gcc 4.5 has a very nice 64bit math emulation :-)
 */
-//#define BENCHMARK 1
+#define BENCHMARK 1
 
-#include "config.h"
 #include <xine/xine_internal.h>
 #include <xine/xineutils.h>
 #include <xine/post.h>
@@ -57,7 +60,7 @@
 #define GOOM_HEIGHT 240
 
 /* colorspace conversion methods */
-static const char* goom_csc_methods[]={
+static const char* const goom_csc_methods[]={
   "Fast but not photorealistic",
   "Slow but looks better",
   "Mostly fast and good quality",
@@ -71,8 +74,10 @@ typedef struct post_class_goom_s post_class_goom_t;
 struct post_class_goom_s {
   post_class_t class;
 
-  post_plugin_goom_t *ip;
-  xine_t             *xine;
+  xine_t *xine;
+  int width, height;
+  int fps;
+  int csc_method;
 };
 
 struct post_plugin_goom_s {
@@ -97,10 +102,8 @@ struct post_plugin_goom_s {
   int channels;
   int sample_rate;
   int samples_per_frame;
-  int width, height;
   int width_back, height_back;
   double ratio;
-  int fps;
   int csc_method;
 
 
@@ -109,7 +112,7 @@ struct post_plugin_goom_s {
 
 
   yuv_planes_t yuv;
-  void *rgb2yuy2;
+  rgb2yuy2_t *rgb2yuy2;
 
   /* frame skipping */
   int skip_frame;
@@ -120,22 +123,6 @@ struct post_plugin_goom_s {
 };
 
 
-/* plugin class initialization function */
-static void *goom_init_plugin(xine_t *xine, void *);
-
-
-/* plugin catalog information */
-static const post_info_t goom_special_info = {
-  XINE_POST_TYPE_AUDIO_VISUALIZATION
-};
-
-const plugin_info_t xine_plugin_info[] EXPORTED = {
-  /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 10, "goom", XINE_VERSION_CODE, &goom_special_info, &goom_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
-};
-
-
 /* plugin class functions */
 static post_plugin_t *goom_open_plugin(post_class_t *class_gen, int inputs,
 					 xine_audio_port_t **audio_target,
@@ -157,87 +144,67 @@ static void goom_port_put_buffer (xine_audio_port_t *this, audio_buffer_t *buf,
 
 static void fps_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   post_class_goom_t *class = (post_class_goom_t*) data;
-
-  if(class->ip) {
-    post_plugin_goom_t *this = class->ip;
-
-    if (cfg->num_value < 1)
-      cfg->num_value = 1;
-
-    this->fps = cfg->num_value;
-
-    if(this->sample_rate)
-      this->samples_per_frame = this->sample_rate / this->fps;
-  }
+  class->fps = cfg->num_value < 1 ? 1
+             : cfg->num_value > 50 ? 50
+             : cfg->num_value;
 }
 
 static void width_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   post_class_goom_t *class = (post_class_goom_t*) data;
-
-  if(class->ip) {
-    post_plugin_goom_t *this = class->ip;
-    this->width = cfg->num_value;
-  }
+  class->width = cfg->num_value;
 }
 
 static void height_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   post_class_goom_t *class = (post_class_goom_t*) data;
-
-  if(class->ip) {
-    post_plugin_goom_t *this = class->ip;
-    this->height = cfg->num_value;
-  }
+  class->height = cfg->num_value;
 }
 
 static void csc_method_changed_cb(void *data, xine_cfg_entry_t *cfg) {
   post_class_goom_t *class = (post_class_goom_t*) data;
-
-  if(class->ip) {
-    post_plugin_goom_t *this = class->ip;
-    this->csc_method = cfg->num_value;
-  }
+  class->csc_method = cfg->num_value;
 }
 
-static void *goom_init_plugin(xine_t *xine, void *data)
-{
-  post_class_goom_t *this = calloc(1, sizeof(post_class_goom_t));
+static void *goom_init_plugin (xine_t *xine, const void *data) {
   config_values_t   *cfg;
-
+  post_class_goom_t *this = calloc (1, sizeof (*this));
   if (!this)
     return NULL;
 
+  (void)data;
+
   this->class.open_plugin     = goom_open_plugin;
   this->class.identifier      = "goom";
   this->class.description     = N_("What a GOOM");
   this->class.dispose         = goom_class_dispose;
-  this->ip                    = NULL;
   this->xine                  = xine;
 
   cfg = xine->config;
 
-  cfg->register_num (cfg, "effects.goom.fps", FPS,
-                                 _("frames per second to generate"),
-                                 _("With more frames per second, the animation will get "
-				   "smoother and faster, but will also require more CPU power."),
-				 10, fps_changed_cb, this);
+  this->fps = cfg->register_num (cfg, "effects.goom.fps", FPS,
+    _("frames per second to generate"),
+    _("With more frames per second, the animation will get "
+      "smoother and faster, but will also require more CPU power."),
+    10, fps_changed_cb, this);
+  this->fps = this->fps < 1 ? 1
+            : this->fps > 50 ? 50
+            : this->fps;
 
-  cfg->register_num (cfg, "effects.goom.width", GOOM_WIDTH,
-                                   _("goom image width"),
-				   _("The width in pixels of the image to be generated."),
-                                   10, width_changed_cb, this);
+  this->width = cfg->register_num (cfg, "effects.goom.width", GOOM_WIDTH,
+    _("goom image width"),
+    _("The width in pixels of the image to be generated."),
+    10, width_changed_cb, this);
 
-  cfg->register_num (cfg, "effects.goom.height", GOOM_HEIGHT,
-                                    _("goom image height"),
-				    _("The height in pixels of the image to be generated."),
-                                    10, height_changed_cb, this);
+  this->height = cfg->register_num (cfg, "effects.goom.height", GOOM_HEIGHT,
+    _("goom image height"),
+    _("The height in pixels of the image to be generated."),
+    10, height_changed_cb, this);
 
-
-  cfg->register_enum (cfg, "effects.goom.csc_method", 0,
-                           goom_csc_methods,
-                           _("colour space conversion method"),
-                           _("You can choose the colour space conversion method used by goom.\n"
-			     "The available selections should be self-explaining."),
-			   20, csc_method_changed_cb, this);
+  this->csc_method = cfg->register_enum (cfg, "effects.goom.csc_method", 0,
+    (char **)goom_csc_methods,
+    _("colour space conversion method"),
+    _("You can choose the colour space conversion method used by goom.\n"
+      "The available selections should be self-explaining."),
+    20, csc_method_changed_cb, this);
 
   return &this->class;
 }
@@ -253,44 +220,25 @@ static post_plugin_t *goom_open_plugin(post_class_t *class_gen, int inputs,
   post_out_t         *output;
   post_out_t         *outputv;
   post_audio_port_t  *port;
-  xine_cfg_entry_t    fps_entry, width_entry, height_entry, csc_method_entry;
 
   if (!this || !video_target || !video_target[0] || !audio_target || !audio_target[0]) {
     free(this);
     return NULL;
   }
 
+  (void)inputs;
+
   _x_post_init(&this->post, 1, 0);
 
-  /*
-   * Lookup config entries.
-   */
   this->class = class;
-  class->ip   = this;
   this->vo_port = video_target[0];
 
   this->metronom = _x_metronom_init(1, 0, class->xine);
 
   lprintf("goom_open_plugin\n");
 
-  if(xine_config_lookup_entry(class->xine, "effects.goom.fps",
-                              &fps_entry))
-    fps_changed_cb(class, &fps_entry);
-
-  if(xine_config_lookup_entry(class->xine, "effects.goom.width",
-                              &width_entry))
-    width_changed_cb(class, &width_entry);
-
-  if(xine_config_lookup_entry(class->xine, "effects.goom.height",
-                              &height_entry))
-    height_changed_cb(class, &height_entry);
-
-  if(xine_config_lookup_entry(class->xine, "effects.goom.csc_method",
-                              &csc_method_entry))
-    csc_method_changed_cb(class, &csc_method_entry);
-
-  this->width_back  = this->width;
-  this->height_back = this->height;
+  this->width_back  = class->width;
+  this->height_back = class->height;
 
   srand((unsigned int)time((time_t *)NULL));
   this->goom = goom_init (this->width_back, this->height_back);
@@ -323,7 +271,7 @@ static post_plugin_t *goom_open_plugin(post_class_t *class_gen, int inputs,
   this->rgb2yuy2 = rgb2yuy2_alloc (10, "bgra");
 #endif
 #ifdef BENCHMARK
-  this->benchmark_frames = 0;
+  this->benchmark_frames = 200 - 1;
   this->benchmark_min = 10000000;
   this->benchmark_time = 0;
 #endif
@@ -335,14 +283,7 @@ static void goom_class_dispose(post_class_t *class_gen)
 {
   post_class_goom_t  *this = (post_class_goom_t*) class_gen;
 
-  this->xine->config->unregister_callback(this->xine->config,
-					  "effects.goom.fps");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "effects.goom.width");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "effects.goom.height");
-  this->xine->config->unregister_callback(this->xine->config,
-					  "effects.goom.csc_method");
+  this->xine->config->unregister_callbacks (this->xine->config, NULL, NULL, this, sizeof (*this));
 
   free(class_gen);
 }
@@ -352,10 +293,9 @@ static void goom_dispose(post_plugin_t *this_gen)
 {
   post_plugin_goom_t *this   = (post_plugin_goom_t *)this_gen;
 
-  rgb2yuy2_free (this->rgb2yuy2);
-
   if (_x_post_dispose(this_gen)) {
-    this->class->ip = NULL;
+
+    rgb2yuy2_free (this->rgb2yuy2);
 
     goom_close(this->goom);
 
@@ -401,9 +341,9 @@ static int goom_port_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
 
   this->channels = _x_ao_mode2channels(mode);
   this->sample_rate = rate;
-  this->samples_per_frame = rate / this->fps;
+  this->samples_per_frame = rate / this->class->fps;
   this->data_idx = 0;
-  init_yuv_planes(&this->yuv, this->width, this->height);
+  init_yuv_planes(&this->yuv, this->width_back, this->height_back);
   this->skip_frame = 0;
 
   this->do_samples_skip = 0;
@@ -412,6 +352,12 @@ static int goom_port_open(xine_audio_port_t *port_gen, xine_stream_t *stream,
   (this->vo_port->open) (this->vo_port, XINE_ANON_STREAM);
   this->metronom->set_master(this->metronom, stream->metronom);
 
+#ifdef BENCHMARK
+  this->benchmark_frames = 200 - 1;
+  this->benchmark_min = 10000000;
+  this->benchmark_time = 0;
+#endif
+
   return (port->original_port->open) (port->original_port, stream, bits, rate, mode );
 }
 
@@ -540,7 +486,7 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
 
       if (!this->skip_frame) {
 #ifdef BENCHMARK
-        int elapsed;
+        int elapsed = 0;
 #endif
         /* Try to be fast */
         goom_frame = (uint8_t *)goom_update (this->goom, this->data, 0, 0, NULL, NULL);
@@ -549,9 +495,11 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
         goom_frame_end = goom_frame + 4 * (this->width_back * this->height_back);
 
 #ifdef BENCHMARK
-        elapsed = -now ();
+        if (this->benchmark_frames >= 0)
+          elapsed = -now ();
 #endif
 
+        this->csc_method = this->class->csc_method;
         if (this->csc_method == 2) {
 
           if (!frame->proc_slice || (frame->height & 15)) {
@@ -573,8 +521,9 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
               frame->proc_slice (frame, sptr);
             }
           }
-
-        } else if ((this->csc_method == 1) &&
+        }
+#if defined(ARCH_X86)
+        else if ((this->csc_method == 1) &&
             (xine_mm_accel() & MM_ACCEL_X86_MMX)) {
           int plane_ptr = 0;
 
@@ -594,8 +543,9 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
           }
 
           yuv444_to_yuy2(&this->yuv, frame->base[0], frame->pitches[0]);
-
-        } else {
+        }
+#endif /* ARCH_X86 */
+        else {
 
           while (goom_frame < goom_frame_end) {
             uint8_t r1, g1, b1, r2, g2, b2;
@@ -629,13 +579,16 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
         }
 
 #ifdef BENCHMARK
-        elapsed += now ();
-        if (this->benchmark_frames < 200) {
+        if (this->benchmark_frames >= 0) {
+          elapsed += now ();
           this->benchmark_time += elapsed;
-          if (elapsed < this->benchmark_min) this->benchmark_min = elapsed;
-          this->benchmark_frames++;
-          if (this->benchmark_frames == 200) printf ("goom: csc_method %d min %d us avg %d us\n",
-            this->csc_method, this->benchmark_min, this->benchmark_time / 200);
+          if (elapsed < this->benchmark_min)
+            this->benchmark_min = elapsed;
+          if (--this->benchmark_frames < 0) {
+            xprintf (this->class->xine, XINE_VERBOSITY_DEBUG,
+              "goom: csc_method %d min %d us avg %d us\n",
+              this->csc_method, this->benchmark_min, this->benchmark_time / 200);
+          }
         }
 #endif
 
@@ -650,18 +603,29 @@ static void goom_port_put_buffer (xine_audio_port_t *port_gen,
 
       frame->free(frame);
 
-      width  = this->width;
-      height = this->height;
+      width  = this->class->width;
+      height = this->class->height;
       if ((width != this->width_back) || (height != this->height_back)) {
         goom_close(this->goom);
-        this->goom = goom_init (this->width, this->height);
+        this->goom = goom_init (width, height);
         this->width_back = width;
         this->height_back = height;
         this->ratio = (double)width/(double)height;
         free_yuv_planes(&this->yuv);
-        init_yuv_planes(&this->yuv, this->width, this->height);
+        init_yuv_planes(&this->yuv, width, height);
       }
     }
   }
   }
 }
+
+/* plugin catalog information */
+static const post_info_t goom_special_info = {
+  .type = XINE_POST_TYPE_AUDIO_VISUALIZATION,
+};
+
+const plugin_info_t xine_plugin_info[] EXPORTED = {
+  /* type, API, "name", version, special_info, init_function */
+  { PLUGIN_POST | PLUGIN_MUST_PRELOAD, 10, "goom", XINE_VERSION_CODE, &goom_special_info, &goom_init_plugin },
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
+};
