@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2017 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * Copyright (C) Christian Vogler
  *               cvogler@gradient.cis.upenn.edu - December 2001
@@ -225,7 +225,7 @@ static const int  rowdata[] = {10, -1, 0, 1, 2, 3, 11, 12, 13, 14, 4, 5, 6,
 			 7, 8, 9};
 /* FIXME: do real ™ (U+2122) */
 /* Code 182 must be mapped as a musical note ('♪', U+266A) in the caption font */
-static const char specialchar[] = {
+static const uint8_t specialchar[] = {
   174 /* ® */, 176 /* ° */, 189 /* ½ */, 191 /* ¿ */,
   'T' /* ™ */, 162 /* ¢ */, 163 /* £ */, 182 /* ¶ => ♪ */,
   224 /* à */, TRANSP_SPACE,232 /* è */, 226 /* â */,
@@ -835,6 +835,7 @@ static void ccmem_init(cc_memory_t *this)
 static void ccmem_exit(cc_memory_t *this)
 {
 /*FIXME: anything to deallocate?*/
+  (void)this;
 }
 
 
@@ -1187,6 +1188,8 @@ static void cc_decode_PAC(cc_decoder_t *this, int channel,
 static void cc_decode_ext_attribute(cc_decoder_t *this, int channel,
 				    uint8_t c1, uint8_t c2)
 {
+  (void)c1;
+  (void)c2;
   cc_set_channel(this, channel);
 }
 
@@ -1196,6 +1199,7 @@ static void cc_decode_special_char(cc_decoder_t *this, int channel,
 {
   cc_buffer_t *buf;
 
+  (void)c1;
   cc_set_channel(this, channel);
   buf = active_ccbuffer(this);
 #ifdef LOG_DEBUG
@@ -1211,6 +1215,7 @@ static void cc_decode_midrow_attr(cc_decoder_t *this, int channel,
   cc_buffer_t *buf;
   cc_attribute_t attr;
 
+  (void)c1;
   cc_set_channel(this, channel);
   buf = active_ccbuffer(this);
   if (c2 < 0x2e) {
@@ -1240,6 +1245,7 @@ static void cc_decode_misc_control_code(cc_decoder_t *this, int channel,
   printf("cc_decoder: decode_misc: decoding %x %x\n", c1, c2);
 #endif
 
+  (void)c1;
   cc_set_channel(this, channel);
 
   switch (c2) {          /* 0x20 <= c2 <= 0x2f */
@@ -1301,6 +1307,7 @@ static void cc_decode_tab(cc_decoder_t *this, int channel,
 {
   cc_buffer_t *buf;
 
+  (void)c1;
   cc_set_channel(this, channel);
   buf = active_ccbuffer(this);
   ccbuf_tab(buf, c2 & 0x3);
