@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2009 the xine project
+ * Copyright (C) 2000-2018 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -44,20 +44,15 @@
 #include "mpeg2_internal.h"
 #include <xine/buffer.h>
 
-typedef struct {
-  video_decoder_class_t   decoder_class;
-} mpeg2_class_t;
-
 
 typedef struct mpeg2dec_decoder_s {
   video_decoder_t  video_decoder;
   mpeg2dec_t       mpeg2;
-  mpeg2_class_t   *class;
   xine_stream_t   *stream;
 } mpeg2dec_decoder_t;
 
 static void mpeg2dec_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
-  mpeg2dec_decoder_t *this = (mpeg2dec_decoder_t *) this_gen;
+  mpeg2dec_decoder_t *this = xine_container_of(this_gen, mpeg2dec_decoder_t, video_decoder);
 
   lprintf ("decode_data, flags=0x%08x ...\n", buf->decoder_flags);
 
@@ -86,7 +81,7 @@ static void mpeg2dec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 }
 
 static void mpeg2dec_flush (video_decoder_t *this_gen) {
-  mpeg2dec_decoder_t *this = (mpeg2dec_decoder_t *) this_gen;
+  mpeg2dec_decoder_t *this = xine_container_of(this_gen, mpeg2dec_decoder_t, video_decoder);
 
   lprintf ("flush\n");
 
@@ -94,20 +89,20 @@ static void mpeg2dec_flush (video_decoder_t *this_gen) {
 }
 
 static void mpeg2dec_reset (video_decoder_t *this_gen) {
-  mpeg2dec_decoder_t *this = (mpeg2dec_decoder_t *) this_gen;
+  mpeg2dec_decoder_t *this = xine_container_of(this_gen, mpeg2dec_decoder_t, video_decoder);
 
   mpeg2_reset (&this->mpeg2);
 }
 
 static void mpeg2dec_discontinuity (video_decoder_t *this_gen) {
-  mpeg2dec_decoder_t *this = (mpeg2dec_decoder_t *) this_gen;
+  mpeg2dec_decoder_t *this = xine_container_of(this_gen, mpeg2dec_decoder_t, video_decoder);
 
   mpeg2_discontinuity (&this->mpeg2);
 }
 
 static void mpeg2dec_dispose (video_decoder_t *this_gen) {
 
-  mpeg2dec_decoder_t *this = (mpeg2dec_decoder_t *) this_gen;
+  mpeg2dec_decoder_t *this = xine_container_of(this_gen, mpeg2dec_decoder_t, video_decoder);
 
   lprintf ("close\n");
 
@@ -121,7 +116,10 @@ static void mpeg2dec_dispose (video_decoder_t *this_gen) {
 static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stream_t *stream) {
   mpeg2dec_decoder_t *this ;
 
+  (void)class_gen;
   this = (mpeg2dec_decoder_t *) calloc(1, sizeof(mpeg2dec_decoder_t));
+  if (!this)
+    return NULL;
 
   this->video_decoder.decode_data         = mpeg2dec_decode_data;
   this->video_decoder.flush               = mpeg2dec_flush;
@@ -129,7 +127,6 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   this->video_decoder.discontinuity       = mpeg2dec_discontinuity;
   this->video_decoder.dispose             = mpeg2dec_dispose;
   this->stream                            = stream;
-  this->class                             = (mpeg2_class_t *) class_gen;
   this->mpeg2.stream = stream;
 
   mpeg2_init (&this->mpeg2, stream->video_out);
@@ -142,18 +139,19 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 /*
  * mpeg2 plugin class
  */
-static void *init_plugin (xine_t *xine, void *data) {
+static void *init_plugin (xine_t *xine, const void *data) {
 
-  mpeg2_class_t *this;
+  static const video_decoder_class_t decode_video_mpeg2_class = {
+    .open_plugin     = open_plugin,
+    .identifier      = "mpeg2dec",
+    .description     = N_("mpeg2 based video decoder plugin"),
+    .dispose         = NULL,
+  };
 
-  this = (mpeg2_class_t *) calloc(1, sizeof(mpeg2_class_t));
+  (void)xine;
+  (void)data;
 
-  this->decoder_class.open_plugin     = open_plugin;
-  this->decoder_class.identifier      = "mpeg2dec";
-  this->decoder_class.description     = N_("mpeg2 based video decoder plugin");
-  this->decoder_class.dispose         = default_video_decoder_class_dispose;
-
-  return this;
+  return (void *)&decode_video_mpeg2_class;
 }
 /*
  * exported plugin catalog entry
@@ -162,12 +160,12 @@ static void *init_plugin (xine_t *xine, void *data) {
 static const uint32_t supported_types[] = { BUF_VIDEO_MPEG, 0 };
 
 static const decoder_info_t dec_info_mpeg2 = {
-  supported_types,     /* supported types */
-  7                    /* priority        */
+  .supported_types = supported_types,
+  .priority        = 7,
 };
 
 const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_VIDEO_DECODER, 19, "mpeg2", XINE_VERSION_CODE, &dec_info_mpeg2, init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
