@@ -587,7 +587,7 @@ static void handle_header(mmal_decoder_t *this, buf_element_t *buf)
 
 static void mmal_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 {
-  mmal_decoder_t *this = (mmal_decoder_t *) this_gen;
+  mmal_decoder_t *this = xine_container_of(this_gen, mmal_decoder_t, video_decoder);
 
   if (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_SPECIAL | BUF_FLAG_COLOR_MATRIX | BUF_FLAG_STDHEADER)) {
     if (buf->decoder_flags & (BUF_FLAG_PREVIEW | BUF_FLAG_SPECIAL)) {
@@ -642,14 +642,14 @@ static void mmal_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 
 static void mmal_flush (video_decoder_t *this_gen)
 {
-  mmal_decoder_t *this = (mmal_decoder_t *) this_gen;
+  mmal_decoder_t *this = xine_container_of(this_gen, mmal_decoder_t, video_decoder);
 
   send_frames(this);
 }
 
 static void mmal_reset (video_decoder_t *this_gen)
 {
-  mmal_decoder_t *this = (mmal_decoder_t *) this_gen;
+  mmal_decoder_t *this = xine_container_of(this_gen, mmal_decoder_t, video_decoder);
 
   free_input_buffer(this);
 
@@ -673,14 +673,15 @@ static void mmal_reset (video_decoder_t *this_gen)
 
 static void mmal_discontinuity (video_decoder_t *this_gen)
 {
-  mmal_decoder_t *this = (mmal_decoder_t *) this_gen;
+  mmal_decoder_t *this = xine_container_of(this_gen, mmal_decoder_t, video_decoder);
+
   send_input_buffer(this);
   this->discontinuity = 1;
 }
 
 static void mmal_dispose (video_decoder_t *this_gen)
 {
-  mmal_decoder_t *this = (mmal_decoder_t *) this_gen;
+  mmal_decoder_t *this = xine_container_of(this_gen, mmal_decoder_t, video_decoder);
 
   free_input_buffer(this);
 
@@ -729,6 +730,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 {
   mmal_decoder_t *this;
   MMAL_STATUS_T   status;
+  const char *codec = NULL;
 
   bcm_host_init();
 
@@ -779,19 +781,19 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 
   switch (video_type) {
     case BUF_VIDEO_H264:
-      _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "H.264");
+      codec = "H.264";
       input->format->encoding = MMAL_ENCODING_H264;
       break;
     case BUF_VIDEO_VC1:
-      _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "VC-1");
+      codec = "VC-1";
       input->format->encoding = MMAL_FOURCC('V','C','-','1');
       break;
     case BUF_VIDEO_MPEG:
-      _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "MPEG");
+      codec = "MPEG";
       input->format->encoding = MMAL_ENCODING_MP2V;
       break;
     case BUF_VIDEO_JPEG:
-      _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, "JPEG");
+      codec = "JPEG";
       input->format->encoding = MMAL_ENCODING_JPEG;
       break;
     default:
@@ -801,6 +803,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
       mmal_dispose(&this->video_decoder);
       return (video_decoder_t *)1;
   }
+  _x_meta_info_set_utf8(this->stream, XINE_META_INFO_VIDEOCODEC, codec);
 
   if (video_type == BUF_VIDEO_H264) {
     MMAL_PARAMETER_BOOLEAN_T param;
@@ -817,8 +820,8 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
   status = mmal_port_format_commit(input);
   if (status != MMAL_SUCCESS) {
     xprintf(this->stream->xine, XINE_VERBOSITY_LOG, LOG_MODULE": "
-            "failed to commit input format: %s (%d)\n",
-            mmal_status_to_string(status), status);
+            "failed to commit input format %s: %s (%d)\n",
+            codec, mmal_status_to_string(status), status);
     mmal_dispose(&this->video_decoder);
     return (video_decoder_t *)1;
   }
